package es.optsicom.problem.gop;


import java.util.Arrays;

import es.optsicom.lib.Solution;
import es.optsicom.problem.gop.function.Function;

public class GOPSolution extends Solution<GOPInstance> {

	private double[] variables;
	private GOPInstance instance;
	private double weight;
	private boolean calculated = false;
	
	public GOPSolution(GOPInstance instance) {
		this(instance, new double[instance.getDimension()]);
	}
	
	public GOPSolution(GOPInstance instance, double[] variables) {
		this.instance = instance;
		this.variables = variables;
	}
	
	private GOPSolution(GOPInstance instance, double[] variables, double weight) {
		this(instance, variables);
		this.weight = weight;
		this.calculated = true;
	}

	@Override
	public void calculatePreciseWeight() {
		instance.evaluate(this);
	}

	@Override
	public Solution<GOPInstance> createCopy() {
		if(calculated) {
			return new GOPSolution(instance, Arrays.copyOf(variables, variables.length), weight);
		}
		return new GOPSolution(instance, Arrays.copyOf(variables, variables.length));
	}

	@Override
	public Object getInfoToSave() {
		return Arrays.toString(variables);
	}

	@Override
	public GOPInstance getInstance() {
		return instance;
	}

	@Override
	public double getWeight() {
		if(!calculated) {
			calculatePreciseWeight();
		}
		return weight;
	}

	@Override
	public boolean isBetterThan(Solution<GOPInstance> aSolution) {
		return instance.getProblem().getQualityComparator().compare(this, aSolution) > 0;
	}

	public double get(int i) {
		return variables[i];
	}

	public void set(int var, double value) {
//		if(value < this.instance.getFunction().getLowerBound() || value > this.instance.getFunction().getUpperBound()) {
//			throw new Error("Value out of bounds: " + value + ", Bounds [" + this.instance.getFunction().getLowerBound() + "," + this.instance.getFunction().getUpperBound() + "]");
//			System.err.println("Unfeasible solution");
//		}
		variables[var] = value;
		calculated = false;
	}

	public boolean isFeasible() {
		Function f = instance.getFunction();
		for(int var = 0; var < this.instance.getDimension(); var++) {
			if(this.variables[var] < f.getLowerBound() || this.variables[var] > f.getUpperBound()) {
				return false;
			}
		}
		return true;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#hashCode()
	 */
	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + Arrays.hashCode(variables);
		return result;
	}

	/* (non-Javadoc)
	 * @see java.lang.Object#equals(java.lang.Object)
	 */
	@Override
	public boolean equals(Object obj) {
		if (this == obj) {
			return true;
		}
		if (obj == null) {
			return false;
		}
		if (!(obj instanceof GOPSolution)) {
			return false;
		}
		GOPSolution other = (GOPSolution) obj;
		if (!Arrays.equals(variables, other.variables)) {
			return false;
		}
		return true;
	}

	public void setWeight(double weight) {
		this.weight = weight;
		calculated = true;
	}

	public double[] getVariables() {
		// Defensive copy
		double[] copy = Arrays.copyOf(variables, variables.length);
		return copy;
	}
}
