package es.optsicom.problem.gop.analysis;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import es.optsicom.lib.InstanceDescription;
import es.optsicom.lib.analysis.AbstractAnalyzer;
import es.optsicom.lib.analysis.table.Cell;
import es.optsicom.lib.analysis.table.CellFormat;
import es.optsicom.lib.analysis.table.ComplexTable;
import es.optsicom.lib.analysis.table.NumberFormat;
import es.optsicom.lib.analysis.table.SheetTable;
import es.optsicom.lib.analysis.table.SimpleSheetCreator;
import es.optsicom.lib.analysis.table.TextSpaceSheetFormatter;
import es.optsicom.lib.analysis.table.TextTabSheetFormatter;
import es.optsicom.lib.analysis.table.NumberFormat.NumberType;
import es.optsicom.lib.experiment.Event;
import es.optsicom.lib.experiment.Execution;
import es.optsicom.lib.experiment.IMExecutions;
import es.optsicom.lib.experiment.MethodDescription;
import es.optsicom.lib.tablecreator.InstanceTableCreator;
import es.optsicom.lib.tablecreator.LastEventRP;
import es.optsicom.lib.tablecreator.RawProcessor;
import es.optsicom.lib.tablecreator.SummarizeColumn;
import es.optsicom.lib.tablecreator.SummarizeRawProcessor;
import es.optsicom.lib.tablecreator.ValueProvider;
import es.optsicom.lib.tablecreator.LastEventRP.Source;
import es.optsicom.lib.util.BestMode;
import es.optsicom.lib.util.Log;
import es.optsicom.lib.util.RelativizeMode;
import es.optsicom.lib.util.SummarizeMode;

/**
 * A flexible analyzer that permits configuring Raw Processors that are used to 
 * build the analysis.
 * 
 * @author Patxi Gortázar
 *
 */
public class FlexibleAnalyzer extends AbstractAnalyzer {

	List<SummarizeRawProcessor> rawProcessors = new ArrayList<SummarizeRawProcessor>();
	
	public FlexibleAnalyzer(BestMode bestMode, SummarizeRawProcessor...gapRP2s) {
		super(bestMode);
		rawProcessors.addAll(Arrays.asList(gapRP2s));
	}

	@Override
	protected void buildAnalysis(List<IMExecutions> imExecutionsList) {
		SheetTable table = new SheetTable(imExecutionsList.size()+2, 1 + rawProcessors.size()*imExecutionsList.get(0).getMethods().size());
		
		// Column titles
		int column = 1;
		for(MethodDescription md : imExecutionsList.get(0).getMethods()) {
			table.setCell(0, column, new Cell(md.getId()));
			for(SummarizeRawProcessor rp : rawProcessors) {
				table.setCell(1, column, new Cell(rp.getCookedEventsNames().get(0)));
				column++;
			}
		}
		
		int row = 2;
		for(IMExecutions instanceMethodsExecs : imExecutionsList) {
			InstanceDescription instance = instanceMethodsExecs.getInstanceDescription();
			table.setCell(row, 0, new Cell(instance.getId()));
			Log.debugln(instance.getId());
			int col = 1;
			for(MethodDescription method : instanceMethodsExecs.getExecsMap().keySet()) {
				List<Execution> methodExecutions = instanceMethodsExecs.getExecsMap().get(method);
				// De estas ejecuciones queremos sacar el max, min, avg, median
				for(SummarizeRawProcessor rp : rawProcessors) {
					double[] minValues = rp.cookEvents(methodExecutions);
					table.setCell(row, col, new Cell(minValues[0], new NumberFormat(NumberType.DECIMAL)));
					col++;
				}
			}
			row++;
		}
		
		TextSpaceSheetFormatter f = new TextSpaceSheetFormatter();
		f.format(table);
		System.out.println(f.getFormattedTable());

		TextTabSheetFormatter f2 = new TextTabSheetFormatter();
		f2.format(table);
		System.out.println(f2.getFormattedTable());

	}
	
}
