package es.optsicom.problem.gop.analysis;

import java.util.Arrays;
import java.util.List;

import es.optsicom.lib.experiment.Event;
import es.optsicom.lib.experiment.Execution;
import es.optsicom.lib.tablecreator.RawProcessor;
import es.optsicom.lib.tablecreator.SummarizeRawProcessor;
import es.optsicom.lib.tablecreator.ValueProvider;

public class GapRP2 extends SummarizeRawProcessor {

	private ValueProvider vp;

	public GapRP2(ValueProvider vp) {
		this.vp = vp;
	}

	@Override
	public double[] cookEvents(Execution exec) {
		double value = (Double) exec.getLastEventValue(Event.OBJ_VALUE_EVENT);
		String instanceName = (String) exec.getInstance().getProperties().get("filename");
		String canonicalName = instanceName.substring(instanceName.lastIndexOf("/")+1);
		double bestValue = (Double)vp.getValue(canonicalName);
		double avgError = Math.abs(value-bestValue);
		
		// From the organizers: "Regarding the precision of the results, it must be taken into account
		// that, if a floating-point type is used for the representation of the
		// solutions (as the double type in C/C++), there is a threshold in the
		// precision of the solutions that, if exceeded, cannot guarantee the
		// reliability of the precision obtained, due to cumulated computational
		// errors. Therefore, it is suggested to approximate all the average errors
		// below 1e-14 to 0.0 in order to allow a fair comparison between the
		// algorithms and extract significant conclusions about their scalability."
		if(avgError < 1.e-14) {
			avgError = 0.0;
		}
		
		return new double[] {avgError};
	}

	@Override
	public List<String> getCookedEventsNames() {
		return Arrays.asList("GAP");
	}

}
