package es.optsicom.problem.gop.approx.improvement;

import java.util.ArrayList;
import java.util.List;

import es.optsicom.lib.util.CollectionsUtil;
import es.optsicom.lib.util.RandomList;
import es.optsicom.problem.gop.GOPSolution;

/**
 * No hace el line search completo del lower al upper, sino que sólo considera numsteps puntos.
 * 
 * @author patxi
 *
 */
public class DynamicOneVarLineSearch extends OneVarLineSearch {
	
	public DynamicOneVarLineSearch() {
		super(ImprovementMode.FIRST);
	}
	
	@Override
	protected List<Double> getPointsToEvaluate(GOPSolution solution, int var, double step) {
		
		double stepSize = solution.getInstance().getStepSize() * solution.getInstance().getBoundsWidth();
		int numSteps = solution.getInstance().getNumSteps();
		
		List<Double> points = new ArrayList<Double>();
		double varValue = solution.get(var);
		for(int i = 0; i < numSteps / 2; i++) {
			varValue -= stepSize;
			if(varValue < solution.getInstance().getFunction().getLowerBound()) {
				break;
			}
			points.add(varValue);
		}
		
		varValue = solution.get(var);
		for(int i = 0; i < numSteps / 2; i++) {
			varValue += stepSize;
			if(varValue > solution.getInstance().getFunction().getUpperBound()) {
				break;
			}
			points.add(varValue);
		}
		
		return points;
	}
}
