package es.optsicom.problem.gop.approx.improvement;

import java.util.ArrayList;
import java.util.List;

import es.optsicom.lib.experiment.CurrentExperiment;
import es.optsicom.lib.util.RandomList;
import es.optsicom.problem.gop.GOPInstance;
import es.optsicom.problem.gop.GOPProblem;
import es.optsicom.problem.gop.GOPSolution;

public class OneVarLineSearch implements IOneVarLineSearch {
	
	public enum ImprovementMode {
		FIRST, BEST
	}

	private ImprovementMode improvementMode = ImprovementMode.BEST;
	
	public OneVarLineSearch() {
		this(ImprovementMode.BEST);
	}
	
	public OneVarLineSearch(ImprovementMode impMode) {
		this.improvementMode = impMode;
	}

	public void performLineSearch(GOPSolution solution, int var, double step) {
		
		List<Double> valuesToEvaluate = getPointsToEvaluate(solution, var, step);
		RandomList<Double> unorderedValues = RandomList.create(valuesToEvaluate);
		
		// TO annotate # evaluations
		int numEvaluationsBefore = solution.getInstance().getNumEvaluations();
		
		double bestValue = solution.getWeight();
		double bestVarValue = solution.get(var);
		for(double point : unorderedValues) {
			solution.set(var, point);
			if(solution.getWeight() < bestValue) {
				
				// First improvement stops when an improvement can be made:
				if(improvementMode == ImprovementMode.FIRST) {
//					solution.set(var, point);
					return;
				}
				
				bestVarValue = point;
				bestValue = solution.getWeight();
			}
		}
		
		// Annotate #evaluations
		if(GOPInstance.SAVE_EVALUATION_EVENTS) {
			CurrentExperiment.addEvent(GOPProblem.LINE_SEARCH_EVALUATIONS, solution.getInstance().getNumEvaluations() - numEvaluationsBefore);
		}
		
		solution.set(var, bestVarValue);
	}

	protected List<Double> getPointsToEvaluate(GOPSolution solution, int var, double step) {
		double lower = solution.getInstance().getFunction().getLowerBound();
		double upper = solution.getInstance().getFunction().getUpperBound();
		
		double varValue = lower;
		List<Double> points = new ArrayList<Double>();
		while(varValue < upper) {
			if(varValue == solution.get(var)) {
				varValue += step;
				continue;
			}
			points.add(varValue);
			varValue += step;
		}
		
		return points;
	}
}
